#include <LedControl.h>
#define DIN_PIN 4
#define CLK_PIN 2
#define LOAD_PIN 3

LedControl lc = LedControl(DIN_PIN, CLK_PIN, LOAD_PIN, 1);

const byte IMAGES[][8] = {
  {
    0b00000000,
    0b00100100,
    0b00100100,
    0b00000000,
    0b01000010,
    0b00111100,
    0b00000000,
    0b00000000
  }, {
    0b00000000,
    0b00100000,
    0b00100000,
    0b00000010,
    0b01000100,
    0b00111000,
    0b00000000,
    0b00000000
  }
};
const int IMAGES_SIZE = sizeof(IMAGES) / 8;

const uint64_t TEXT[] = {
  0x6666667e66663c00, // A
  0x66361e3e66663e00, // R
  0x3e66666666663e00, // D
  0x7c66666666666600, // U
  0x3c18181818183c00, // I
  0x666666767e6e6600, // N
  0x3c66666666663c00  // O
};
const int TEXT_SIZE = sizeof(TEXT) / 8;

void setup() {
  lc.shutdown(0, false);
  lc.setIntensity(0, 7);
  lc.clearDisplay(0);
}

void loop() {
  pixelAnimation(1000);
  rowColumnAnimation(1500);
  bitmapAnimation(1000);
  textAnimation(3500);
}

void pixelAnimation(unsigned long duration) {
  for (int i = 1; i >= 0; i--) {
    for (int j = 0; j < 8; j++) {
      if (j < 3) {
        lc.setLed(0, j+3, j, i);
      } else {
        lc.setLed(0, 7-j, j, i);
      }
      delay(duration / 16);
    }
  }
}

void rowColumnAnimation(unsigned long duration) {
  for (int i = 0; i < 8; i++) {
    lc.setRow(0, i, 0xFF);
    delay(duration / 16);
  }
  for (int i = 0; i < 8; i++) {
    lc.setColumn(0, i, 0x00);
    delay(duration / 16);
  }
}

void bitmapAnimation(unsigned long duration) {
  for (int i = 0; i < 3; i++) {
    for (int j = 0; j < 8; j++) {
      lc.setRow(0, j, IMAGES[i%IMAGES_SIZE][j]);
    }
    delay(duration / 4);
  }
  delay(duration / 4);
  lc.clearDisplay(0);
}

void textAnimation(unsigned long duration) {
  for (int i = 0; i < TEXT_SIZE; i++) {
    displayImage(TEXT[i]);
    delay(duration / TEXT_SIZE);
  }
  lc.clearDisplay(0);
}

void displayImage(uint64_t image) {
  for (int i = 0; i < 8; i++) {
    byte row = (image >> i * 8) & 0xFF;
    for (int j = 0; j < 8; j++) {
      lc.setLed(0, i, j, bitRead(row, j));
    }
  }
}